@@ -3,12 +3,13 @@ import numpy as np
 
 from string import Template
 
+
 from ..arc import to_threepoint
 from ..entities import Line, Arc, BSpline, Text
 
+from ... import resources
 from ...constants import log
 from ...constants import tol_path as tol
-from ...resources import get_resource
 from ... import util
 from ... import grouping
 
@@ -60,7 +61,7 @@ XRECORD_MAX_INDEX = 368
 
 # get the TEMPLATES for exporting DXF files
 TEMPLATES = {k: Template(v) for k, v in json.loads(
-    get_resource('dxf.json.template')).items()}
+    resources.get('dxf.json.template')).items()}
 
 
 def load_dxf(file_obj, **kwargs):
@@ -165,99 +166,29 @@ def load_dxf(file_obj, **kwargs):
         # in my otherwise normal polygon", it's like SVG arc
         # flags but somehow even more annoying
         if '42' in e:
-            # from Autodesk reference:
-            # The bulge is the tangent of one fourth the included
-            # angle for an arc segment, made negative if the arc
-            # goes clockwise from the start point to the endpoint.
-            # A bulge of 0 indicates a straight segment, and a
-            # bulge of 1 is a semicircle.
-            log.debug('polyline bulge: {}'.format(e['42']))
-            # the actual bulge float values
+            # get the actual bulge float values
             bulge = np.array(e['42'], dtype=np.float64)
-
             # what position were vertices stored at
             vid = np.nonzero(chunk[:, 0] == '10')[0]
             # what position were bulges stored at in the chunk
             bid = np.nonzero(chunk[:, 0] == '42')[0]
-
+            # filter out endpoint bulge if we're not closed
             if not is_closed:
                 bid_ok = bid < vid.max()
                 bid = bid[bid_ok]
                 bulge = bulge[bid_ok]
-
             # which vertex index is bulge value associated with
             bulge_idx = np.searchsorted(vid, bid)
-            # use bulge to calculate included angle of the arc
-            angle = np.arctan(bulge) * 4.0
-
-            # the indexes making up a bulged segment
-            tid = np.column_stack((bulge_idx, bulge_idx - 1))
-
-            # if it's a closed segment modulus to start vertex
-            if is_closed:
-                tid %= len(lines)
-            # the vector connecting the two ends of the arc
-            vector = lines[tid[:, 0]] - lines[tid[:, 1]]
-            # the length of the connector segment
-            length = (np.linalg.norm(vector, axis=1))
-
-            # perpendicular vectors by crossing vector with Z
-            perp = np.cross(
-                np.column_stack((vector, np.zeros(len(vector)))),
-                np.ones((len(vector), 3)) * [0, 0, 1])
-            # strip the zero Z
-            perp = util.unitize(perp[:, :2])
-
-            # midpoint of each line
-            midpoint = lines[tid].mean(axis=1)
-
-            # calculate the signed radius of each arc segment
-            radius = (length / 2.0) / np.sin(angle / 2.0)
-
-            # offset magnitude to point on arc
-            offset = radius - np.cos(angle / 2) * radius
-
-            # convert each arc to three points:
-            # start, any point on arc, end
-            three = np.column_stack((
-                lines[tid[:, 0]],
-                midpoint + perp * offset.reshape((-1, 1)),
-                lines[tid[:, 1]])).reshape((-1, 3, 2))
-
-            # if we're in strict mode make sure our arcs
-            # have the same magnitude as the input data
-            if True or tol.strict:
-                from .. import arc as arcmod
-                check_angle = [arcmod.arc_center(i)['span']
-                               for i in three]
-                assert np.allclose(np.abs(angle),
-                                   np.abs(check_angle))
-
-                check_radii = [arcmod.arc_center(i)['radius']
-                               for i in three]
-                assert np.allclose(check_radii, np.abs(radius))
-
-            # if there are unconsumed line
-            # segments add them to drawing
-            if (len(lines) - 1) > len(bulge):
-                # indexes of line segments
-                existing = util.stack_lines(np.arange(len(lines)))
-                # remove line segments replaced with arcs
-                for line_idx in grouping.boolean_rows(
-                        existing,
-                        np.sort(tid, axis=1),
-                        np.setdiff1d):
-                    # add a single line entity and vertices
-                    entities.append(Line(
-                        points=np.arange(2) + len(vertices),
-                        **polyinfo))
-                    vertices.extend(lines[line_idx])
-            # add the three point arcs to the result
-            for arc_points in three:
-                entities.append(Arc(
-                    points=np.arange(3) + len(vertices),
-                    **polyinfo))
-                vertices.extend(arc_points)
+            # convert stupid bulge to Line/Arc entities
+            v, e = bulge_to_arcs(lines=lines,
+                                 bulge=bulge,
+                                 bulge_idx=bulge_idx,
+                                 is_closed=is_closed)
+            for i in e:
+                # offset added entities by current vertices length
+                i.points += len(vertices)
+            vertices.extend(v)
+            entities.extend(e)
             # done with this polyline
             return
 
@@ -308,36 +239,30 @@ def load_dxf(file_obj, **kwargs):
         """
         Convert a DXF TEXT entity into a native text entity.
         """
-        if '50' in e:
-            # rotation angle converted to radians
-            angle = np.radians(float(e['50']))
-        else:
-            # otherwise no rotation
-            angle = 0.0
-
         # text with leading and trailing whitespace removed
         text = e['1'].strip()
-
-        # height of text
-        if '40' in e:
+        # try getting optional height of text
+        try:
             height = float(e['40'])
-        else:
+        except BaseException:
             height = None
-
+        try:
+            # rotation angle converted to radians
+            angle = np.radians(float(e['50']))
+        except BaseException:
+            # otherwise no rotation
+            angle = 0.0
         # origin point
-        origin = np.array([e['10'],
-                           e['20']]).astype(np.float64)
-
-        # an origin- relative point (so transforms work)
+        origin = np.array(
+            [e['10'], e['20']], dtype=np.float64)
+        # an origin-relative point (so transforms work)
         vector = origin + [np.cos(angle), np.sin(angle)]
-
         # try to extract a (horizontal, vertical) text alignment
         align = ['center', 'center']
         try:
             align[0] = ['left', 'center', 'right'][int(e['72'])]
         except BaseException:
             pass
-
         # append the entity
         entities.append(Text(origin=len(vertices),
                              vector=len(vertices) + 1,
@@ -478,21 +403,39 @@ def load_dxf(file_obj, **kwargs):
         # the end of a polyline
         elif polyline is not None and entity_type == 'SEQEND':
             # pull the geometry information for the entity
-            lines = [[i['10'], i['20']]
-                     for i in polyline[1:]]
+            lines = np.array([[i['10'], i['20']]
+                              for i in polyline[1:]],
+                             dtype=np.float64)
+
             # check for a closed flag on the polyline
             if '70' in polyline[0]:
                 # flag is bit- coded integer
                 flag = int(polyline[0]['70'])
                 # first bit represents closed
-                if bool(flag & 1):
-                    lines.append(lines[0])
-            # create a single Line entity
-            entities.append(Line(
-                points=np.arange(len(lines)) + len(vertices),
-                **info(dict(polyline[0]))))
-            # add the vertices to our collection
-            vertices.extend(lines)
+                is_closed = bool(flag & 1)
+                if is_closed:
+                    lines = np.vstack((lines, lines[:1]))
+
+            # get the index of each bulged vertices
+            bulge_idx = np.array([i for i, e in enumerate(polyline)
+                                  if '42' in e],
+                                 dtype=np.int64)
+            # get the actual bulge value
+            bulge = np.array([float(e['42'])
+                              for i, e in enumerate(polyline)
+                              if '42' in e],
+                             dtype=np.float64)
+            # convert bulge to new entities
+            v, e = bulge_to_arcs(lines=lines,
+                                 bulge=bulge,
+                                 bulge_idx=bulge_idx,
+                                 is_closed=is_closed)
+            for i in e:
+                # offset entities by existing vertices
+                i.points += len(vertices)
+            vertices.extend(v)
+            entities.extend(e)
+
             # we no longer have an active polyline
             polyline = None
         elif entity_type == 'TEXT':
@@ -535,15 +478,19 @@ def load_dxf(file_obj, **kwargs):
 
 def export_dxf(path, layers=None):
     """
-    Export a 2D path object to a DXF file
+    Export a 2D path object to a DXF file.
 
     Parameters
     ----------
-    path: trimesh.path.path.Path2D
+    path : trimesh.path.path.Path2D
+      Input geometry to export
+    layers : None, set or iterable
+      If passed only export the layers specified
 
     Returns
     ----------
-    export: str, path formatted as a DXF file
+    export : str
+      Path formatted as a DXF file
     """
 
     def format_points(points,
@@ -572,7 +519,8 @@ def export_dxf(path, layers=None):
           Points formatted with group code
         """
         points = np.asanyarray(points, dtype=np.float64)
-        three = util.three_dimensionalize(points, return_2D=False)
+        # get points in 3D
+        three = util.stack_3D(points)
         if increment:
             group = np.tile(
                 np.arange(len(three), dtype=np.int).reshape((-1, 1)),
@@ -584,7 +532,7 @@ def export_dxf(path, layers=None):
         if as_2D:
             group = group[:, :2]
             three = three[:, :2]
-
+        # join into result string
         packed = '\n'.join('{:d}\n{:.12f}'.format(g, v)
                            for g, v in zip(group.reshape(-1),
                                            three.reshape(-1)))
@@ -605,19 +553,13 @@ def export_dxf(path, layers=None):
         subs : dict
           Has keys 'COLOR', 'LAYER', 'NAME'
         """
+        # TODO : convert RGBA entity.color to index
         subs = {'COLOR': 255,  # default is ByLayer
                 'LAYER': 0,
                 'NAME': str(id(entity))[:16]}
-
-        if hasattr(entity, 'color'):
-            # all colors must be integers between 0-255
-            color = str(entity.color)
-            if str.isnumeric(color):
-                subs['COLOR'] = int(color) % 256
-
         if hasattr(entity, 'layer'):
-            subs['LAYER'] = str(entity.layer)
-
+            # make sure layer name is forced into ASCII
+            subs['LAYER'] = util.to_ascii(entity.layer)
         return subs
 
     def convert_line(line, vertices):
@@ -725,19 +667,21 @@ def export_dxf(path, layers=None):
         """
         Convert a Text entity to DXF string.
         """
+        # start with layer info
         sub = entity_info(txt)
-
         # get the origin point of the text
-        sub['ORIGIN'] = format_points(vertices[[txt.origin]],
-                                      increment=False)
+        sub['ORIGIN'] = format_points(
+            vertices[[txt.origin]], increment=False)
         # rotation angle in degrees
         sub['ANGLE'] = np.degrees(txt.angle(vertices))
         # actual string of text with spaces escaped
-        sub['TEXT'] = txt.text.replace(' ', _SAFESPACE)
+        # force into ASCII to avoid weird encoding issues
+        sub['TEXT'] = txt.text.replace(' ', _SAFESPACE).encode(
+            'ascii', errors='ignore').decode('ascii')
         # height of text
         sub['HEIGHT'] = txt.height
-
-        return TEMPLATES['text'].substitute(sub)
+        result = TEMPLATES['text'].substitute(sub)
+        return result
 
     def convert_generic(entity, vertices):
         """
@@ -764,20 +708,19 @@ def export_dxf(path, layers=None):
                 continue
         if name in conversions:
             converted = conversions[name](e, path.vertices).strip()
-            # only save if we converted something
             if len(converted) > 0:
+                # only save if we converted something
                 collected.append(converted)
         else:
             log.debug('Entity type %s not exported!', name)
 
+    # join all entities into one string
     entities_str = '\n'.join(collected)
-
     hsub = {'BOUNDS_MIN': format_points([path.bounds[0]]),
             'BOUNDS_MAX': format_points([path.bounds[1]]),
             'LUNITS': '1'}
     if path.units in _UNITS_TO_DXF:
         hsub['LUNITS'] = _UNITS_TO_DXF[path.units]
-
     # sections of the DXF
     header = TEMPLATES['header'].substitute(hsub)
     # entities section
@@ -790,23 +733,21 @@ def export_dxf(path, layers=None):
     # although Draftsight, LibreCAD, and Inkscape don't care
     # what a giant legacy piece of shit
     # strip out all leading and trailing whitespace
-    sections = [i.strip() for i in [header,
-                                    entities,
-                                    footer]
+    sections = [i.strip() for i in
+                [header, entities, footer]
                 if len(i) > 0]
-
+    # create the joined string blob
     blob = '\n'.join(sections).replace(_SAFESPACE, ' ')
-
     # run additional self- checks
     if tol.strict:
         # check that every line pair is (group code, value)
         lines = str.splitlines(str(blob))
-
         # should be even number of lines
         assert (len(lines) % 2) == 0
-
         # group codes should all be convertible to int and positive
         assert all(int(i) >= 0 for i in lines[::2])
+        # make sure we didn't slip any unicode in there
+        blob.encode('ascii')
 
     return blob
 
@@ -835,6 +776,147 @@ def load_dwg(file_obj, **kwargs):
     return result
 
 
+def bulge_to_arcs(lines,
+                  bulge,
+                  bulge_idx,
+                  is_closed=False,
+                  metadata=None):
+    """
+    Polylines can have "vertex bulge," which means the polyline
+    has an arc tangent to segments, rather than meeting at a
+    vertex.
+
+    From Autodesk reference:
+    The bulge is the tangent of one fourth the included
+    angle for an arc segment, made negative if the arc
+    goes clockwise from the start point to the endpoint.
+    A bulge of 0 indicates a straight segment, and a
+    bulge of 1 is a semicircle.
+
+    Parameters
+    ----------------
+    lines : (n, 2) float
+      Polyline vertices in order
+    bulge : (m,) float
+      Vertex bulge value
+    bulge_idx : (m,) float
+      Which index of lines is bulge associated with
+    is_closed : bool
+      Is segment closed
+    metadata : None, or dict
+      Entity metadata to add
+
+    Returns
+    ---------------
+    vertices : (a, 2) float
+      New vertices for poly-arc
+    entities : (b,) entities.Entity
+      New entities, either line or arc
+    """
+    # make sure lines are 2D array
+    lines = np.asanyarray(lines, dtype=np.float64)
+
+    # make sure inputs are numpy arrays
+    bulge = np.asanyarray(bulge, dtype=np.float64)
+    bulge_idx = np.asanyarray(bulge_idx, dtype=np.int64)
+
+    # filter out zero- bulged polylines
+    ok = np.abs(bulge) > 1e-5
+    bulge = bulge[ok]
+    bulge_idx = bulge_idx[ok]
+
+    # metadata to apply to new entities
+    if metadata is None:
+        metadata = {}
+
+    # if there's no bulge, just return the input curve
+    if len(bulge) == 0:
+        index = np.arange(len(lines))
+        # add a single line entity and vertices
+        entities = [Line(index, **metadata)]
+        return lines, entities
+
+    # use bulge to calculate included angle of the arc
+    angle = np.arctan(bulge) * 4.0
+    # the indexes making up a bulged segment
+    tid = np.column_stack((bulge_idx, bulge_idx - 1))
+    # if it's a closed segment modulus to start vertex
+    if is_closed:
+        tid %= len(lines)
+
+    # the vector connecting the two ends of the arc
+    vector = lines[tid[:, 0]] - lines[tid[:, 1]]
+
+    # the length of the connector segment
+    length = (np.linalg.norm(vector, axis=1))
+
+    # perpendicular vectors by crossing vector with Z
+    perp = np.cross(
+        np.column_stack((vector, np.zeros(len(vector)))),
+        np.ones((len(vector), 3)) * [0, 0, 1])
+    # strip the zero Z
+    perp = util.unitize(perp[:, :2])
+
+    # midpoint of each line
+    midpoint = lines[tid].mean(axis=1)
+
+    # calculate the signed radius of each arc segment
+    radius = (length / 2.0) / np.sin(angle / 2.0)
+
+    # offset magnitude to point on arc
+    offset = radius - np.cos(angle / 2) * radius
+
+    # convert each arc to three points:
+    # start, any point on arc, end
+    three = np.column_stack((
+        lines[tid[:, 0]],
+        midpoint + perp * offset.reshape((-1, 1)),
+        lines[tid[:, 1]])).reshape((-1, 3, 2))
+
+    # if we're in strict mode make sure our arcs
+    # have the same magnitude as the input data
+    if tol.strict:
+        from ..arc import arc_center
+        check_angle = [arc_center(i)['span']
+                       for i in three]
+        assert np.allclose(np.abs(angle),
+                           np.abs(check_angle))
+
+        check_radii = [arc_center(i)['radius']
+                       for i in three]
+        assert np.allclose(check_radii, np.abs(radius))
+
+    # collect new entities and vertices
+    entities, vertices = [], []
+    # add the entities for each new arc
+    for arc_points in three:
+        entities.append(Arc(
+            points=np.arange(3) + len(vertices),
+            **metadata))
+        vertices.extend(arc_points)
+
+    # if there are unconsumed line
+    # segments add them to drawing
+    if (len(lines) - 1) > len(bulge):
+        # indexes of line segments
+        existing = util.stack_lines(np.arange(len(lines)))
+        # remove line segments replaced with arcs
+        for line_idx in grouping.boolean_rows(
+                existing,
+                np.sort(tid, axis=1),
+                np.setdiff1d):
+            # add a single line entity and vertices
+            entities.append(Line(
+                points=np.arange(2) + len(vertices),
+                **metadata))
+            vertices.extend(lines[line_idx].copy())
+
+    # make sure vertices are clean numpy array
+    vertices = np.array(vertices, dtype=np.float64)
+
+    return vertices, entities
+
+
 def get_key(blob, field, code):
     """
     Given a loaded (n, 2) blob and a field name
@@ -919,8 +1001,8 @@ def _teigha_convert(data, extension='dwg'):
         raise ValueError('conversion using Teigha failed!')
 
     # load converted file into a string
-    with open(name_result, 'r') as f:
-        converted = f.read()
+    with open(name_result, 'rb') as f:
+        converted = f.read().decode(errors='ignore')
 
     # remove the temporary directories
     shutil.rmtree(dir_out)
